@@ -200,7 +200,7 @@ gnome_sort_cmd (GtkWidget *widget, WPanel *panel)
         GtkWidget *menu;
         GtkWidget *menu_item;
 	GtkWidget *cbox1, *cbox2;
-	sortfn *sfn;
+	sortfn *sfn = NULL;
 
 	sort_box = gnome_dialog_new (_("Sort By"), GNOME_STOCK_BUTTON_OK, 
 				     GNOME_STOCK_BUTTON_CANCEL, NULL);
@@ -541,3 +541,108 @@ gnome_select_all_cmd (GtkWidget *widget, WPanel *panel)
 	paint_panel (panel);
 	do_refresh ();
 }
+void
+gnome_filter_cmd (GtkWidget *widget, WPanel *panel)
+{
+	GtkWidget *filter_dlg;
+	GtkWidget *entry;
+	GtkWidget *label;
+	gchar *text1, *text2, *text3;
+	GList *list;
+
+	filter_dlg = gnome_dialog_new (_("Set Filter"), GNOME_STOCK_BUTTON_OK, 
+				       GNOME_STOCK_BUTTON_CANCEL, NULL);
+	if (easy_patterns) {
+		text1 = "mc_filter_globs";
+		text3 = _("Show all files");
+		if (panel->filter && (strcmp (panel->filter, "*")))
+			text2 = panel->filter;
+		else
+			text2 = NULL;
+	} else {
+		text1 = ("mc_filter_regexps");
+		text3 = _(".");
+		if (!panel->filter)
+			text2 = NULL;
+		else
+			text2 = panel->filter;
+	}
+	entry = gnome_entry_new (text1);
+	gnome_entry_load_history (GNOME_ENTRY (entry));
+	
+	if (text2) {
+		gtk_entry_set_text (GTK_ENTRY (gnome_entry_gtk_entry (GNOME_ENTRY (entry))), text2);
+		gnome_entry_prepend_history (GNOME_ENTRY (entry), FALSE, text3);
+	} else
+		gtk_entry_set_text (GTK_ENTRY (gnome_entry_gtk_entry (GNOME_ENTRY (entry))), text3);
+
+	if (easy_patterns)
+		label = gtk_label_new (_("Enter a filter here for files in the panel view.\n\nFor example:\n*.gif will show just gif images"));
+	else
+		label = gtk_label_new (_("Enter a Regular Expression to filter files in the panel view."));
+	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (filter_dlg)->vbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (filter_dlg)->vbox), entry, FALSE, FALSE, 0);
+	gtk_widget_show_all (GNOME_DIALOG (filter_dlg)->vbox);
+	switch (gnome_dialog_run (GNOME_DIALOG (filter_dlg))) {
+	case 0:
+		gtk_widget_hide (filter_dlg);
+		if (panel->filter) {
+			g_free (panel->filter);
+			panel->filter = NULL;
+		}
+		panel->filter = g_strdup (gtk_entry_get_text (GTK_ENTRY (gnome_entry_gtk_entry (GNOME_ENTRY (entry)))));
+		if (!strcmp (_("Show all files"), panel->filter)) {
+			g_free (panel->filter);
+			panel->filter = NULL;
+			gtk_entry_set_text (GTK_ENTRY (gnome_entry_gtk_entry (GNOME_ENTRY (entry))),
+					    "");
+			gtk_label_set_text (GTK_LABEL (panel->status), _("Show all files"));
+		} else if (!strcmp ("*", panel->filter)) {
+			g_free (panel->filter);
+			panel->filter = NULL;
+			gtk_label_set_text (GTK_LABEL (panel->status), _("Show all files"));
+		} else
+			gtk_label_set_text (GTK_LABEL (panel->status), panel->filter);
+		
+		gnome_entry_save_history (GNOME_ENTRY (entry));
+		reread_cmd ();
+		x_filter_changed (panel);
+		break;
+	}
+	gtk_widget_destroy (filter_dlg);
+}
+void
+gnome_open_files (GtkWidget *widget, WPanel *panel)
+{
+	GList *later = NULL;
+	GList *now;
+	gint i;
+
+	/* FIXME: this is the easy way to do things.  We want the
+	 * hard way sometime. */
+	for (i = 0; i < panel->count; i++) {
+		if (panel->dir.list [i].f.marked)
+			if (!do_enter_on_file_entry ((panel->dir.list) + i))
+				later = g_list_prepend (later, panel->dir.list + i);
+	}
+#if 0
+	/* This is sorta ugly.  Should we just skip these? There should be a better way. */
+	for (now = later; now; now = now->next) {
+		gchar *command;
+		command = input_expand_dialog (_(" Open with..."),
+					       _("Enter extra arguments:"), (WPanel *) now->data);
+		if (!command)
+			/* we break out. */
+			break;
+		execute (command);
+		free (command);
+	}
+#endif
+	g_list_free (later);
+	   
+}
+
