@@ -460,12 +460,25 @@ internal_select_item (GtkWidget *file_list, WPanel *panel, int row)
 
 	select_item (panel);
 }
-
+static int
+panel_file_list_press_row (GtkWidget *file_list, GdkEvent *event, WPanel *panel)
+{
+	/* FIXME: This is still very broken. */
+	if (event->type == GDK_BUTTON_PRESS && event->button.button == 3) {
+		gint row, column;
+		gtk_clist_get_selection_info (GTK_CLIST (file_list),
+					      event->button.x, event->button.y,
+					      &row, &column);
+		gpopup_do_popup ((GdkEventButton *) event, panel, NULL, row, panel->dir.list[row].fname);
+	}
+	return TRUE;
+}
 static void
 panel_file_list_select_row (GtkWidget *file_list, int row, int column, GdkEvent *event, WPanel *panel)
 {
 	int current_selection = panel->selected;
-
+	char *fullname;
+	
 	if (!event) {
 		internal_select_item (file_list, panel, row);
 		return;
@@ -478,19 +491,21 @@ panel_file_list_select_row (GtkWidget *file_list, int row, int column, GdkEvent
 
 		switch (event->button.button) {
 		case 1:
-			if (!(event->button.state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)))
-				break;
-
-			/* fallback if shift-click is pressed */
-			do_file_mark_range (panel, row, current_selection);
+			if (!(event->button.state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK))){
+				do_file_mark (panel, row, !panel->dir.list[row].f.marked);
+			} else {
+				/* if shift-click is pressed */
+				do_file_mark_range (panel, row, current_selection);
+			}
 			break;
 
 		case 2:
-			do_file_mark (panel, row, !panel->dir.list[row].f.marked);
-			break;
-
-		case 3:
-			gpopup_do_popup ((GdkEventButton *) event, panel, NULL, row, panel->dir.list[row].fname);
+			if (S_ISDIR (panel->dir.list [current_selection].buf.st_mode) ||
+			    panel->dir.list [current_selection].f.link_to_dir){
+				fullname = concat_dir_and_file (panel->cwd, panel->dir.list [current_selection].fname);
+				new_panel_at (fullname);
+				free (fullname);
+			}
 			break;
 		}
 
@@ -769,7 +784,8 @@ panel_icon_list_drag_data_received (GtkWidget          *widget,
 	GnomeIconList *gil = GNOME_ICON_LIST (widget);
 	char *dir;
 	int idx;
-
+	gboolean reload;
+	
 	idx = gnome_icon_list_get_icon_at (gil, x, y);
 	if (idx == -1)
 		dir = g_strdup (panel->cwd);
@@ -781,11 +797,13 @@ panel_icon_list_drag_data_received (GtkWidget          *widget,
 			dir = g_strdup (panel->cwd);
 	}
 
-	gdnd_drop_on_directory (context, selection_data, dir);
+	reload = gdnd_drop_on_directory (context, selection_data, dir);
 	free (dir);
 
-	update_one_panel_widget (panel, 0, UP_KEEPSEL);
-	panel_update_contents (panel);
+	if (reload){
+		update_one_panel_widget (panel, 0, UP_KEEPSEL);
+		panel_update_contents (panel);
+	}
 }
 
 /**
@@ -1139,7 +1157,12 @@ panel_create_file_list (WPanel *panel)
 	gtk_signal_connect (GTK_OBJECT (file_list), "select_row",
 			    GTK_SIGNAL_FUNC (panel_file_list_select_row),
 			    panel);
-
+	gtk_signal_connect (GTK_OBJECT (file_list), "button_press_event",
+			    GTK_SIGNAL_FUNC (panel_file_list_press_row),
+			    panel);
+	gtk_clist_set_button_actions (GTK_CLIST (file_list), 1, GTK_BUTTON_SELECTS | GTK_BUTTON_DRAGS);
+	gtk_clist_set_button_actions (GTK_CLIST (file_list), 2, GTK_BUTTON_SELECTS);
+	
 	/* Set up drag and drop */
 
 	load_dnd_icons ();
@@ -1148,6 +1171,11 @@ panel_create_file_list (WPanel *panel)
 			   drop_types, ELEMENTS (drop_types),
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 
+	/* Make directories draggable */
+	gtk_drag_source_set (GTK_WIDGET (file_list), GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
+			     drag_types, ELEMENTS (drag_types),
+			     GDK_ACTION_LINK | GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_ASK);
+	
 	gtk_signal_connect (GTK_OBJECT (file_list), "drag_data_get",
 			    GTK_SIGNAL_FUNC (panel_drag_data_get), panel);
 	gtk_signal_connect (GTK_OBJECT (file_list), "drag_data_delete",
@@ -1196,13 +1224,16 @@ panel_icon_list_select_icon (GtkWidget *widget, int index, GdkEvent *event, WPan
 	display_mini_info (panel);
 	execute_hooks (select_file_hook);
 
-	if (event)
-		printf ("Icon list select icon, event->type= %d %d %d\n", event->type, GDK_BUTTON_PRESS, GDK_2BUTTON_PRESS);
 	if (!event)
 		return;
 
 	switch (event->type){
 	case GDK_BUTTON_PRESS:
+		if (event->button.button == 3)
+			gpopup_do_popup ((GdkEventButton *) event, panel, NULL, index, panel->dir.list[index].fname);
+		break;
+
+	case GDK_BUTTON_RELEASE:
 		if (event->button.button == 2){
 			char *fullname;
 
@@ -1214,10 +1245,8 @@ panel_icon_list_select_icon (GtkWidget *widget, int index, GdkEvent *event, WPan
 			}
 			break;
 		} 
-		if (event->button.button == 3)
-			gpopup_do_popup ((GdkEventButton *) event, panel, NULL, index, panel->dir.list[index].fname);
 		break;
-
+		
 	case GDK_2BUTTON_PRESS:
 		if (event->button.button == 1)
 			do_enter (panel);
@@ -1572,13 +1601,14 @@ show_filter_popup (GtkWidget *button, gpointer data)
 void
 display_mini_info (WPanel *panel)
 {
-	GtkLabel *label = GTK_LABEL (panel->ministatus);
+	GnomeAppBar *bar = GNOME_APPBAR (panel->ministatus);
 
 	if (panel->searching) {
 		char *buf;
 
 		buf = g_strdup_printf (_("Search: %s"), panel->search_buffer);
-		gtk_label_set (label, buf);
+		gnome_appbar_pop (bar);
+		gnome_appbar_push (bar, buf);
 		g_free (buf);
 		return;
 	}
@@ -1589,7 +1619,8 @@ display_mini_info (WPanel *panel)
 		buf = g_strdup_printf ((panel->marked == 1) ? _("%s bytes in %d file") : _("%s bytes in %d files"),
 				       size_trunc_sep (panel->total),
 				       panel->marked);
-		gtk_label_set (label, buf);
+		gnome_appbar_pop (bar);
+		gnome_appbar_push (bar, buf);
 		g_free (buf);
 		return;
 	}
@@ -1606,10 +1637,13 @@ display_mini_info (WPanel *panel)
 			link_target [len] = 0;
 			/* FIXME: Links should be handled differently */
 			/* str = copy_strings ("-> ", link_target, NULL); */
-			gtk_label_set (label, " ");
-			   /*free (str); */
-		} else
-			gtk_label_set (label, _("<readlink failed>"));
+			gnome_appbar_pop (bar);
+			gnome_appbar_push (bar, " ");
+			/*free (str); */
+		} else {
+			gnome_appbar_pop (bar);
+			gnome_appbar_push (bar, _("<readlink failed>"));
+		}
 		return;
 	}
 
@@ -1620,12 +1654,14 @@ display_mini_info (WPanel *panel)
 		buffer = xmalloc (len + 2, "display_mini_info");
 		format_file (buffer, panel, panel->selected, panel->estimated_total-2, 0, 1);
 		buffer [len] = 0;
-		gtk_label_set (label, buffer);
+		gnome_appbar_pop (bar);
+		gnome_appbar_push (bar, buffer);
 		free (buffer);
 	}
 	if (panel->list_type == list_icons){
 		if (panel->marked == 0){
-			gtk_label_set (label, " ");
+			gnome_appbar_pop (bar);
+			gnome_appbar_push (bar, " ");
 		}
 	}
 }
@@ -2248,47 +2284,30 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 	gnome_dock_add_item (GNOME_DOCK(GNOME_APP (panel->xwindow)->dock),
 			     GNOME_DOCK_ITEM (dock), GNOME_DOCK_TOP, 1, 0, 0, FALSE);
 	gtk_widget_show_all (dock);
-
-
-	/*
-	 * ministatus
-	 */
-	panel->ministatus = gtk_label_new (" "); /* was a cliplabel */
-	gtk_widget_set_usize (panel->ministatus, 0, -1);
-	gtk_misc_set_alignment (GTK_MISC (panel->ministatus), 0.0, 0.0);
-	gtk_misc_set_padding (GTK_MISC (panel->ministatus), 3, 0);
-	gtk_widget_show (panel->ministatus);
-	gtk_label_set_justify (GTK_LABEL (panel->ministatus), GTK_JUSTIFY_LEFT);
-
-	/*
-	 * The statusbar
-	 * This status bar now holds the  ministatus.
-	 */
-	frame = gtk_frame_new (NULL);
-	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
-	gtk_container_set_border_width (GTK_CONTAINER (frame), 3);
-
-	panel->status = gtk_label_new (""); /* used to be a cliplabel */
-
-
-	/* we set up the status_bar */
-	gtk_misc_set_alignment (GTK_MISC (panel->status), 0.0, 0.5);
-	gtk_misc_set_padding (GTK_MISC (panel->status), 3, 0);
-	gtk_container_add (GTK_CONTAINER (frame), panel->ministatus);
-	gtk_label_set_justify (GTK_LABEL (panel->status), GTK_JUSTIFY_LEFT);
-	gtk_widget_show_all (frame);
-
-
 	panel->view_table = gtk_table_new (1, 1, 0);
 	gtk_widget_show (panel->view_table);
 
+
+	/*
+	 * The status bar.
+	 */
+	panel->ministatus = GNOME_APPBAR(gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER));
+	gnome_app_set_statusbar(GNOME_APP (panel->xwindow), GTK_WIDGET(panel->ministatus));
+	ministatus_box = gtk_frame_new (NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME(ministatus_box), GTK_SHADOW_IN);
+      
+	panel->status = gtk_label_new (_("Show all files"));
+	gtk_misc_set_alignment (GTK_MISC (panel->status), 0.0, 0.0);
+      
+	gtk_box_pack_start (GTK_BOX (panel->ministatus), ministatus_box, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER(ministatus_box), panel->status);
+      
+	gtk_widget_show (ministatus_box);
+	gtk_widget_show (panel->status);
+
 	/*
 	 * Put the icon list and the file listing in a nice frame
 	 */
-	table_frame = gtk_frame_new (NULL);
-	gtk_frame_set_shadow_type (GTK_FRAME (table_frame), GTK_SHADOW_IN);
-	gtk_widget_show (table_frame);
-	gtk_container_add (GTK_CONTAINER (table_frame), panel->view_table);
 
 	/* Add both the icon view and the listing view */
 	gtk_table_attach (GTK_TABLE (panel->view_table), panel->notebook, 0, 1, 0, 1,
@@ -2301,7 +2320,7 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			  0, 0);
 
-	gtk_paned_add2 (GTK_PANED (panel->pane), table_frame);
+	gtk_paned_add2 (GTK_PANED (panel->pane), panel->view_table);
 
 #if 0
 	gtk_table_attach (GTK_TABLE (panel->table), status_line, 0, 1, 0, 1,
@@ -2328,11 +2347,11 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			  0, 0, 0);
 
-#endif
 	gtk_table_attach (GTK_TABLE (panel->table), frame, 0, 1, 3, 4,
 			  GTK_EXPAND | GTK_FILL,
 			  0, 0, 0);
 
+#endif
 	/* Ultra nasty hack: pull the vbox from wdata */
 	vbox =  GTK_WIDGET (panel->widget.wdata);
 
