@@ -82,6 +82,31 @@ gbutton_callback (GtkWidget *w, void *data)
 	}
 }
 
+char *
+stock_from_text (char *text)
+{
+	char *stock;
+		
+	if (strcasecmp (text, _("ok")) == 0)
+		stock = GNOME_STOCK_BUTTON_OK;
+	else if (strcasecmp (text, _("cancel")) == 0)
+		stock = GNOME_STOCK_BUTTON_CANCEL;
+	else if (strcasecmp (text, _("help")) == 0)
+		stock = GNOME_STOCK_BUTTON_HELP;
+	else if (strcasecmp (text, _("yes")) == 0)
+		stock = GNOME_STOCK_BUTTON_YES;
+	else if (strcasecmp (text, _("no")) == 0)
+		stock = GNOME_STOCK_BUTTON_NO;
+	else if (strcasecmp (text, _("exit")) == 0)
+		stock = GNOME_STOCK_BUTTON_CLOSE;
+	else if (strcasecmp (text, _("abort")) == 0)
+		stock = GNOME_STOCK_BUTTON_CANCEL;
+	else
+		stock = 0;
+
+	return stock;
+}
+
 int
 x_create_button (Dlg_head *h, widget_data parent, WButton *b)
 {
@@ -89,22 +114,7 @@ x_create_button (Dlg_head *h, widget_data parent, WButton *b)
 	char *stock;
 	int tag;
 
-	if (strcasecmp (b->text, _("ok")) == 0)
-		stock = GNOME_STOCK_BUTTON_OK;
-	else if (strcasecmp (b->text, _("cancel")) == 0)
-		stock = GNOME_STOCK_BUTTON_CANCEL;
-	else if (strcasecmp (b->text, _("help")) == 0)
-		stock = GNOME_STOCK_BUTTON_HELP;
-	else if (strcasecmp (b->text, _("yes")) == 0)
-		stock = GNOME_STOCK_BUTTON_YES;
-	else if (strcasecmp (b->text, _("no")) == 0)
-		stock = GNOME_STOCK_BUTTON_NO;
-	else if (strcasecmp (b->text, _("exit")) == 0)
-		stock = GNOME_STOCK_BUTTON_CLOSE;
-	else if (strcasecmp (b->text, _("abort")) == 0)
-		stock = GNOME_STOCK_BUTTON_CANCEL;
-	else
-		stock = 0;
+	stock = stock_from_text (b->text);
 
 	if (stock){
 		button = gnome_stock_button (stock);
@@ -324,7 +334,7 @@ x_update_input (WInput *in)
 		in->first = 0;
 	}
 
-	text = GTK_ENTRY (entry)->text;
+	text = gtk_entry_get_text (GTK_ENTRY (entry));
 	
 	if (text && strcmp (text, in->buffer)){
 		gtk_entry_set_text (entry, in->buffer);
@@ -342,14 +352,18 @@ x_update_input (WInput *in)
 #else
 		gtk_widget_draw (GTK_WIDGET (entry), NULL);
 #endif
-		/* This is a private member, hopefully the widget
-                   properly updates itself */
-		gtk_entry_adjust_scroll (GTK_ENTRY (entry));
+		gtk_editable_changed (GTK_EDITABLE (entry));
 		gtk_widget_queue_draw (GTK_WIDGET (entry));
 	}
 }
 
 /* Listboxes */
+static GtkWidget *
+listbox_pull (widget_data data)
+{
+	return GTK_BIN (data)->child;
+}
+
 void
 x_listbox_select_nth (WListbox *l, int nth)
 {
@@ -363,7 +377,7 @@ x_listbox_select_nth (WListbox *l, int nth)
 		return;
 	
 	inside = 1;
-	clist = GTK_CLIST (l->widget.wdata);
+	clist = GTK_CLIST (listbox_pull (l->widget.wdata));
 	
 	gtk_clist_select_row (clist, nth, 0);
 	if (gtk_clist_row_is_visible (clist, nth) != GTK_VISIBILITY_FULL)
@@ -375,7 +389,7 @@ x_listbox_select_nth (WListbox *l, int nth)
 void
 x_listbox_delete_nth (WListbox *l, int nth)
 {
-	gtk_clist_remove (GTK_CLIST (l->widget.wdata), nth);
+	gtk_clist_remove (GTK_CLIST (listbox_pull (l->widget.wdata)), nth);
 }
 
 static void
@@ -425,20 +439,26 @@ listbox_select (GtkWidget *widget, int row, int column, GdkEvent *event, WListbo
 int
 x_create_listbox (Dlg_head *h, widget_data parent, WListbox *l)
 {
-	GtkWidget *listbox;
+	GtkWidget *listbox, *sw;
 	GtkRequisition req;
 	WLEntry *p;
 	int i;
 	
 	listbox = gtk_clist_new (1);
+	sw = gtk_scrolled_window_new (NULL, NULL);
+	gtk_container_add (GTK_CONTAINER (sw), listbox);
+	
 	gtk_clist_set_selection_mode (GTK_CLIST (listbox), GTK_SELECTION_BROWSE);
 	gtk_widget_size_request (listbox, &req);
 	gtk_widget_set_usize (listbox, req.width, req.height + 20*8);
 	gtk_signal_connect (GTK_OBJECT (listbox), "select_row",
 			    GTK_SIGNAL_FUNC (listbox_select), l);
+	l->widget.wdata = (widget_data) sw;
 	gtk_widget_show (listbox);
-	l->widget.wdata = (widget_data) listbox;
 
+	g_warning ("FIXME: actually compute the real size of the listbox");
+	l->height = 8;
+	
 	for (p = l->list, i = 0; i < l->count; i++, p = p->next){
 		char *text [1];
 
@@ -470,7 +490,7 @@ x_list_insert (WListbox *l, WLEntry *p, WLEntry *e)
 		return;
 	}
 	text [0] = e->text;
-	gtk_clist_append (GTK_CLIST (l->widget.wdata), text);
+	gtk_clist_append (GTK_CLIST (listbox_pull (l->widget.wdata)), text);
 }
 
 /* Labels */
