@@ -6,53 +6,48 @@
  * License or the Perl Artistic License.
  */
 
-#include <config.h>
+
+#include "config.h"
+
 #include <stdio.h>
 #include <string.h>
-#ifndef NO_STDLIB_H
-#include <stdlib.h>
+#include <stdarg.h>
+
+#ifdef HAVE_STDLIB_H
+# include <stdlib.h>
 #endif
 
 #include "slang.h"
+#include "_slang.h"
 
 void (*SLang_Error_Routine)(char *);
 void (*SLang_Exit_Error_Hook)(char *);
 volatile int SLang_Error = 0;
 char *SLang_Error_Message;
+volatile int SLKeyBoard_Quit = 0;
 
-void SLang_doerror(char *error)
+void SLang_doerror (char *error)
 {
-   char err[256]; char *str = NULL;
+   char err [1024];
+   char *str = NULL;
 
-   if (!SLang_Error) SLang_Error = UNKNOWN_ERROR;
    *err = 0;
    
-   if (SLang_Error_Message != NULL) str = SLang_Error_Message;
    str = "Slang/Midnight Commander unknown error";
-   SLang_Error_Message = NULL;
-   
+
    sprintf(err, "S-Lang Error: %s", str);
    
    if (SLang_Error_Routine == NULL)
      {
-	if (error != NULL) 
-	  {
-	     fputs(error, stderr);
-	     fputs("\r\n", stderr);
-	  }
-	
-	if (str != error) 
-	  {
-	     fputs(err, stderr);
-	     fputs("\r\n", stderr);
-	  }
+	fputs (err, stderr);
+	fputs("\r\n", stderr);
      }
    else
-     {	if (error != NULL) (*SLang_Error_Routine)(error);
-	if (str != error) (*SLang_Error_Routine)(err);
-     }
+     (*SLang_Error_Routine)(err);
 }
 
+
+
 void SLang_exit_error (char *s)
 {
    if (SLang_Exit_Error_Hook != NULL)
