@@ -65,7 +65,7 @@
 #define HISTORY_SIZE 20
 #define HELP_WINDOW_WIDTH 62
 
-/* "$Id: help.c,v 1.7 1998/12/02 23:44:25 unammx Exp $" */
+/* "$Id: help.c,v 1.8 1999/01/10 08:55:58 unammx Exp $" */
 
 static char *data;		/* Pointer to the loaded data file */
 static int help_lines = 18;	/* Lines in help viewer */
@@ -96,7 +96,7 @@ static int inside_link_area = 0;
 
 static int help_callback (struct Dlg_head *h, int id, int msg);
 
-#ifdef OS2_NT
+#ifdef HAS_ACS_AS_PCCHARS
 struct {
     int acscode;
     int pccode;
