@@ -72,7 +72,7 @@
 #endif
 
 #include <errno.h>
-#ifndef OS2_NT
+#ifndef HAS_NO_GRP_PWD_H
 #    include <pwd.h>
 #endif
 #include <ctype.h>
@@ -128,8 +128,10 @@
 #include "chmod.h"
 #include "chown.h"
 
-#ifdef OS2_NT
+#ifdef NEEDS_IO_H
 #    include <io.h>
+#endif
+#ifdef NEEDS_DRIVE_H
 #    include <drive.h>
 #endif
 
@@ -139,7 +141,7 @@
 #include <glib.h>
 #include "popt.h"
 
-/* "$Id: main.c,v 1.68 1999/01/03 18:34:48 unammx Exp $" */
+/* "$Id: main.c,v 1.71 1999/01/12 09:08:42 unammx Exp $" */
 
 /* When the modes are active, left_panel, right_panel and tree_panel */
 /* Point to a proper data structure.  You should check with the functions */
@@ -2808,8 +2810,34 @@ compatibility_move_mc_files (void)
 }
 #endif
 
+void
+mc_tree_store_load ()
+{
+	char *tree_file;
+	
+	tree_file = concat_dir_and_file (home_dir, MC_TREE);
+	tree_store_init ();
+	tree_store_load (tree_file);
+	free (tree_file);
+}
+
+void
+mc_tree_store_save ()
+{
+	char *tree_file;
+
+	printf ("Saving tree!\n");
+	tree_file = concat_dir_and_file (home_dir, MC_TREE);
+	tree_store_save (tree_file);
+	free (tree_file);
+}
+
 int main (int argc, char *argv [])
 {
+#ifdef HAVE_GNOME
+    /* Just to time things */
+    printf ("GNU Midnight Commander " VERSION "\n");
+#endif
     /* We had LC_CTYPE before, LC_ALL includs LC_TYPE as well */
     setlocale (LC_ALL, "");
     bindtextdomain ("mc", LOCALEDIR);
@@ -2855,6 +2883,8 @@ int main (int argc, char *argv [])
 
     handle_args(argc, argv);
 
+    mc_tree_store_load ();
+
     session_management_setup (argv [0]);
     probably_finish_program ();
 #endif
