@@ -18,7 +18,7 @@
 /* Events received by clients of this library have their coordinates 0 */
 /* based */
 
-/* "$Id: mouse.c,v 1.2 1998/05/20 02:20:18 unammx Exp $" */
+/* "$Id: mouse.c,v 1.3 1999/01/10 08:56:03 unammx Exp $" */
 
 #include <config.h>
 #include <sys/types.h>
@@ -27,7 +27,7 @@
 #endif
 #include <signal.h>	/* For kill() and SIGQUIT */
 #include <fcntl.h>
-#if (!defined(__IBMC__) && !defined(__IBMCPP__)) && !defined(OS2_NT)
+#if (!defined(__IBMC__) && !defined(__IBMCPP__)) && !defined(HAS_NO_TERMIOS_H)
 #    include <termios.h>
 #endif
 #include <malloc.h>
