@@ -129,7 +129,7 @@ slang_intr (int signo)
     slinterrupt = 1;
 }
 
-int
+static int
 interrupts_enabled (void)
 {
     struct sigaction current_act;
@@ -268,7 +268,7 @@ slang_shutdown ()
    sequence. -- Norbert
  */   
    
-void
+static void
 slang_reset_softkeys (void)
 {
     int key;
@@ -351,19 +351,27 @@ int max_index = 0;
 void
 init_pair (int index, char *foreground, char *background)
 {
+    /* hack for transparent background for Eterm, rxvt or else */
+    if (!strcmp(background, "default"))
+       background = NULL;
+    /* if foreground is default, I guess we should use normal fore-color. */
 
     SLtt_set_color (index, "", foreground, background);
     if (index > max_index)
 	max_index = index;
 }
 
-int
+static int
 alloc_color_pair (char *foreground, char *background)
 {
     init_pair (++max_index, foreground, background);
     return max_index;
 }
 
+extern struct colorpair {
+    char *name, *fg, *bg;
+} color_map[];
+
 int
 try_alloc_color_pair (char *fg, char *bg)
 {
@@ -376,7 +384,7 @@ try_alloc_color_pair (char *fg, char *bg)
 	0, 0, 0, 0
     };
 
-    c.index = NORMAL_COLOR;
+    c.index = EDITOR_NORMAL_COLOR_INDEX;
     p = &c;
     for (;;) {
 	if (((fg && p->fg) ? !strcmp (fg, p->fg) : fg == p->fg) != 0
@@ -392,9 +400,10 @@ try_alloc_color_pair (char *fg, char *bg)
     p->fg = fg ? strdup (fg) : 0;
     p->bg = bg ? strdup (bg) : 0;
     if (!fg)
-	fg = "white";
+        /* Index in color_map array = COLOR_INDEX - 1 */
+	fg = color_map[EDITOR_NORMAL_COLOR_INDEX - 1].fg;
     if (!bg)
-	bg = "blue";
+	bg = color_map[EDITOR_NORMAL_COLOR_INDEX - 1].bg;
     p->index = alloc_color_pair (fg, bg);
     return p->index;
 }
