@@ -2,6 +2,7 @@
 #define __VFS_H
 
 #include <config.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
 
@@ -9,7 +10,7 @@
 #include "src/mad.h"
 #endif
 
-#if !defined(SCO_FLAVOR) || !defined(_SYS_SELECT_H)
+#if !defined(SCO_FLAVOR) || !defined(_SYS_SELECT_H) || defined(IS_AIX)
 #	include <sys/time.h>	/* alex: this redefines struct timeval */
 #endif /* SCO_FLAVOR */
 
@@ -241,7 +242,7 @@ struct utimbuf {
 #   define vfs_current_is_tarfs() 0
 #   define vfs_current_is_extfs() 0
 #   define vfs_path(x) x
-#   define vfs_strip_suffix_from_filename (x) strdup(x)
+#   define vfs_strip_suffix_from_filename(x) strdup(x)
 #   define mc_close close
 #   define mc_read read
 #   define mc_write write
@@ -273,7 +274,8 @@ struct utimbuf {
 #   define mc_munmap munmap
 
 #   define mc_ctl(a,b,c) 0
-#   define mc_setctl(a,b,c)
+#   define mc_setctl(a,b,c) 0
+#   define vfs_translate_url strdup(s)
 
 #   define mc_stat stat
 #   define mc_mknod mknod
